@@ -16,6 +16,7 @@ from .token import TOKEN_DDOT
 from .token import TOKEN_DOT_PROPERTY
 from .token import TOKEN_DOUBLE_QUOTE_STRING
 from .token import TOKEN_EQ
+from .token import TOKEN_FAKE_ROOT
 from .token import TOKEN_FALSE
 from .token import TOKEN_FILTER
 from .token import TOKEN_FILTER_CONTEXT
@@ -66,10 +67,34 @@ if TYPE_CHECKING:
 
 
 class Lexer:
-    """Tokenize a JSONPath string."""
+    """Tokenize a JSONPath string.
+
+    Some customization can be achieved by subclassing _Lexer_ and setting
+    class attributes. Then setting `lexer_class` on a `JSONPathEnvironment`.
+
+    Attributes:
+        key_pattern: The regular expression pattern used to match mapping
+            keys/properties.
+        logical_not_pattern: The regular expression pattern used to match
+            logical negation tokens. By default, `not` and `!` are
+            equivalent.
+        logical_and_pattern: The regular expression pattern used to match
+            logical _and_ tokens. By default, `and` and `&&` are equivalent.
+        logical_or_pattern: The regular expression pattern used to match
+            logical _or_ tokens. By default, `or` and `||` are equivalent.
+    """
 
     key_pattern = r"[\u0080-\uFFFFa-zA-Z_][\u0080-\uFFFFa-zA-Z0-9_-]*"
 
+    # `not` or !
+    logical_not_pattern = r"(?:not|!)"
+
+    # && or `and`
+    logical_and_pattern = r"(?:&&|and)"
+
+    # || or `or`
+    logical_or_pattern = r"(?:\|\||or)"
+
     def __init__(self, *, env: JSONPathEnvironment) -> None:
         self.env = env
 
@@ -85,15 +110,6 @@ class Lexer:
             r"(?::\s*(?P<G_LSLICE_STEP>\-?\d*))?"
         )
 
-        # `not` or !
-        self.logical_not_pattern = r"(?:not|!)"
-
-        # && or `and`
-        self.bool_and_pattern = r"(?:&&|and)"
-
-        # || or `or`
-        self.bool_or_pattern = r"(?:\|\||or)"
-
         # /pattern/ or /pattern/flags
         self.re_pattern = r"/(?P<G_RE>.+?)/(?P<G_RE_FLAGS>[aims]*)"
 
@@ -104,6 +120,17 @@ class Lexer:
 
     def compile_rules(self) -> Pattern[str]:
         """Prepare regular expression rules."""
+        env_tokens = [
+            (TOKEN_ROOT, self.env.root_token),
+            (TOKEN_FAKE_ROOT, self.env.fake_root_token),
+            (TOKEN_SELF, self.env.self_token),
+            (TOKEN_KEY, self.env.key_token),
+            (TOKEN_UNION, self.env.union_token),
+            (TOKEN_INTERSECTION, self.env.intersection_token),
+            (TOKEN_FILTER_CONTEXT, self.env.filter_context_token),
+            (TOKEN_KEYS, self.env.keys_selector_token),
+        ]
+
         rules = [
             (TOKEN_DOUBLE_QUOTE_STRING, self.double_quote_pattern),
             (TOKEN_SINGLE_QUOTE_STRING, self.single_quote_pattern),
@@ -114,15 +141,15 @@ class Lexer:
             (TOKEN_FLOAT, r"-?\d+\.\d*(?:e[+-]?\d+)?"),
             (TOKEN_INT, r"-?\d+(?P<G_EXP>e[+\-]?\d+)?\b"),
             (TOKEN_DDOT, r"\.\."),
-            (TOKEN_AND, self.bool_and_pattern),
-            (TOKEN_OR, self.bool_or_pattern),
-            (TOKEN_ROOT, re.escape(self.env.root_token)),
-            (TOKEN_SELF, re.escape(self.env.self_token)),
-            (TOKEN_KEY, re.escape(self.env.key_token)),
-            (TOKEN_UNION, re.escape(self.env.union_token)),
-            (TOKEN_INTERSECTION, re.escape(self.env.intersection_token)),
-            (TOKEN_FILTER_CONTEXT, re.escape(self.env.filter_context_token)),
-            (TOKEN_KEYS, re.escape(self.env.keys_selector_token)),
+            (TOKEN_AND, self.logical_and_pattern),
+            (TOKEN_OR, self.logical_or_pattern),
+            *[
+                (token, re.escape(pattern))
+                for token, pattern in sorted(
+                    env_tokens, key=lambda x: len(x[1]), reverse=True
+                )
+                if pattern
+            ],
             (TOKEN_WILD, r"\*"),
             (TOKEN_FILTER, r"\?"),
             (TOKEN_IN, r"in"),
